<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 3/18/16
 * Time: 5:43 PM
 */

namespace qfq;

const NESTING_TOKEN_OPEN = '#&nesting-open-&#';
const NESTING_TOKEN_CLOSE = '#&nesting-close&#';
const NESTING_TOKEN_LENGTH = 17;

class BodytextParser {


    /**
     * @param $bodytext
     * @return mixed
     */
    public function process($bodytext) {
        $bodytext = $this->trimAndRemoveCommentAndEmptyLine($bodytext);
        // Encrypt double curly braces to prevent false positives with nesting: form = {{form}}\n
        $bodytext = Support::encryptDoubleCurlyBraces($bodytext);
        $bodytext = $this->encryptNestingDelimeter($bodytext);
        $bodytext = $this->joinLine($bodytext);
        $bodytext = $this->unNest($bodytext);
        $bodytext = $this->trimAndRemoveCommentAndEmptyLine($bodytext);
        $bodytext = Support::decryptDoubleCurlyBraces($bodytext);

        if (strpos($bodytext, NESTING_TOKEN_OPEN) !== false) {
            throw new \qfq\UserFormException("Missing close delimiter: $bodytext", ERROR_MISSING_CLOSE_DELIMITER);
        }
        return $bodytext;

    }

    /**
     * Trim all lines, remove all empty lines and  all lines which start with '#'
     * @param $bodytext
     * @return string
     */

    private function trimAndRemoveCommentAndEmptyLine($bodytext) {
        $data = array();

        $src = explode(PHP_EOL, $bodytext);
        foreach ($src as $row) {
            $row = trim($row);
            if ($row === '' || $row[0] === '#') {
                continue;
            }
            $data[] = $row;
        }

        return implode(PHP_EOL, $data);
    }

    /**
     * Encrypt '{\n' and '}\n' by more complex token.
     *
     * @param $bodytext
     * @return mixed
     */
    private function encryptNestingDelimeter($bodytext) {
        // Take care that a trailing '}' will be recognised: add '\n'
        if (substr($bodytext, -1) === '}') {
            $bodytext .= "\n";
        }

        $bodytext = str_replace("{\n", NESTING_TOKEN_OPEN, $bodytext);
        $bodytext = str_replace("}\n", NESTING_TOKEN_CLOSE, $bodytext);
        return $bodytext;
    }

    /**
     * Join lines, which do not begin with '<level>.<keyword>[ ]='
     *
     * @param array $bodytextArray
     * @return string
     */
    private function joinLine($bodytext) {
        $data = array();
        $bodytextArray = explode(PHP_EOL, $bodytext);

        $full = '';
        foreach ($bodytextArray as $row) {
            // Valid 'new line' starts indicators: form, <level>, <level.sublevel>, <level>.<keyword>, {, <level> {, }
            if ((1 === preg_match('/^\s*(\d*(\.)?)*\s*(head|althead|tail|sql|rbeg|rend|renr|rsep|fbeg|fend|fsep|' .
                        TYPO3_FORM . '|' . TYPO3_DEBUG_SHOW_BODY_TEXT . '|' . TYPO3_RECORD_ID . ') *=/', $row))
                || (1 === preg_match('/^\s*(\d*(\.)?)*\s*({|})\s*/', $row))
                || (1 === preg_match('/^\s*(\d+(\.)?)+/', $row))
            ) {

                // if there is already something: save this
                if ($full !== '')
                    $data[] = $full;

                // start new line
                $full = $row;

            } else {

                // continue row: concat
                $full .= ' ' . $row;
            }
        }

        // Save last line
        if ($full !== '')
            $data[] = $full;

        return implode(PHP_EOL, $data);
    }

//PREG_SPLIT_DELIM_CAPTURE

    /**
     * @param $bodytext
     * @return mixed|string
     * @throws UserFormException
     */
    private function unNest($bodytext) {

        // Replace '\{' | '\}' by internal token. All remaining '}' | '{' means: 'nested'
//        $bodytext = str_replace('\{', '#&[_#', $bodytext);
//        $bodytext = str_replace('\}', '#&]_#', $bodytext);
//        $bodytext = Support::encryptDoubleCurlyBraces($bodytext);

        $result = $bodytext;
        $posFirstClose = strpos($result, NESTING_TOKEN_CLOSE);

        while ($posFirstClose !== false) {
            $posMatchOpen = strrpos(substr($result, 0, $posFirstClose), NESTING_TOKEN_OPEN);

            if ($posMatchOpen === false) {
                $result = $this->decryptNestingDelimeter($result);
                throw new \qfq\UserFormException("Missing open delimiter: $result", ERROR_MISSING_OPEN_DELIMITER);
            }

            $pre = substr($result, 0, $posMatchOpen);
            if ($pre === false)
                $pre = '';

            $post = substr($result, $posFirstClose + NESTING_TOKEN_LENGTH);
            if ($post === false)
                $post = '';

            // trim also removes '\n'
            $match = trim(substr($result, $posMatchOpen + NESTING_TOKEN_LENGTH, $posFirstClose - $posMatchOpen - NESTING_TOKEN_LENGTH));

            // "10.sql = SELECT...\n20 {\n
            $levelStartPos = strrpos(trim($pre), PHP_EOL);
            $levelStartPos = ($levelStartPos === false) ? 0 : $levelStartPos + 1;  // Skip PHP_EOL

            $level = trim(substr($pre, $levelStartPos));
//            if($level==='') {
//                $pre=
//            }
            // remove 'level' from last line
            $pre = substr($pre, 0, $levelStartPos);

            // Split nested content in single rows
            $lines = explode(PHP_EOL, $match);
            foreach ($lines as $line) {
                $pre .= $level . '.' . $line . PHP_EOL;
            }

            $result = $pre . $post;
            $posFirstClose = strpos($result, NESTING_TOKEN_CLOSE);
        }

//        $result = str_replace('#&[_#', '{', $result);
//        $result = str_replace('#&]_#', '}', $result);
//        $result = Support::decryptDoubleCurlyBraces($result);

        return $result;
    }

    /**
     * Decrypt complex token by '{\n' and '}\n'
     *
     * @param $bodytext
     * @return mixed
     */
    private function decryptNestingDelimeter($bodytext) {

        $bodytext = str_replace(NESTING_TOKEN_OPEN, "{\n", $bodytext);
        $bodytext = str_replace(NESTING_TOKEN_CLOSE, "}\n", $bodytext);
        return $bodytext;
    }

}