<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 5/4/16
 * Time: 1:34 PM
 */

namespace qfq;


class Session {
    private static $instance = null;
    private static $phpUnit = null;
    private static $sessionLocal = array();

    /**
     * @param bool|false $phpUnit
     * @throws CodeException
     */
    private function __construct($phpUnit = false) {
        if (self::$phpUnit !== null)
            throw new CodeException("Try to set flag phpunit again - that should not happen.", ERROR_CODE_SHOULD_NOT_HAPPEN);

        self::$phpUnit = $phpUnit;

        if (self::$phpUnit) {
            self::$sessionLocal = array();
        } else {
            session_name();
            session_start();
        }
    }

    /**
     * @param bool|false $phpUnit
     * @return null|\qfq\Store
     */
    public static function getInstance($phpUnit = false) {

        // Design Pattern: Singleton
        if (self::$instance === null) {
            self::$instance = new self($phpUnit);
        }

        return self::$instance;
    }

    /**
     * @param $key
     * @return bool
     */
    public static function get($key) {

        if (self::$phpUnit) {
            if (isset(self::$sessionLocal[$key]))
                $value = self::$sessionLocal[$key];
            else
                $value = false;
        } else {
            if (isset($_SESSION[$key]))
                $value = $_SESSION[$key];
            else
                $value = false;

        }

        return $value;
    }

    /**
     * @param $key
     * @param $value
     */
    public static function set($key, $value) {

        if (self::$phpUnit) {
            self::$sessionLocal[$key] = $value;
        } else {
            $_SESSION[$key] = $value;
        }
    }

    /**
     */
    public static function clear() {

        if (self::$phpUnit) {
            self::$sessionLocal = array();
        } else {
            unset($_SESSION);
        }
    }

}