<?php

//namespace qfq;

//use qfq;

require_once(__DIR__ . '/AbstractDatabaseTest.php');
require_once(__DIR__ . '/../../qfq/report/Report.php');
require_once(__DIR__ . '/../../qfq/store/Store.php');
require_once(__DIR__ . '/../../qfq/Evaluate.php');

/**
 * Created by PhpStorm.
 * User: crose
 * Date: 2/2/16
 * Time: 10:07 PM
 */
class ReportTest extends AbstractDatabaseTest {

    /**
     * @var qfq\Evaluate
     */
    private $eval = null;

    /**
     * @var qfq\Report
     */
    private $report = null;

    /**
     *
     */
    public function testReportSingleQuery() {

        //empty
        $result = $this->report->process("");
        $this->assertEquals('', $result);

        // comment
        $result = $this->report->process('# 10.sql = SELECT "Hello World"');
        $this->assertEquals('', $result);

        // sql
        $result = $this->report->process('10.sql = SELECT "Hello World"');
        $this->assertEquals('Hello World', $result);

        // sql, 2 rows
        $result = $this->report->process('10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2');
        $this->assertEquals('DoeJohnSmithJane', $result);

        // sql, 2 rows, head
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>");
        $this->assertEquals('<table>DoeJohnSmithJane', $result);

        // sql, 2 rows, head, tail
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>\n10.tail = </table>");
        $this->assertEquals('<table>DoeJohnSmithJane</table>', $result);

        // sql, 2 rows, head, tail, rbeg
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>\n10.tail = </table>\n10.rbeg = <tr>");
        $this->assertEquals('<table><tr>DoeJohn<tr>SmithJane</table>', $result);

        // sql, 2 rows, head, tail, rbeg, rend
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>\n10.tail = </table>\n10.rbeg = <tr>\n10.rend = </tr>");
        $this->assertEquals('<table><tr>DoeJohn</tr><tr>SmithJane</tr></table>', $result);

        // sql, 2 rows, head, tail, rbeg, rend, fbeg
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>\n10.tail = </table>\n10.rbeg = <tr>\n10.rend = </tr>\n10.fbeg = <td>");
        $this->assertEquals('<table><tr><td>Doe<td>John</tr><tr><td>Smith<td>Jane</tr></table>', $result);

        // sql, 2 rows, head, tail, rbeg, rend, fbeg, fend
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>\n10.tail = </table>\n10.rbeg = <tr>\n10.rend = </tr>\n10.fbeg = <td>\n10.fend = </td>");
        $this->assertEquals('<table><tr><td>Doe</td><td>John</td></tr><tr><td>Smith</td><td>Jane</td></tr></table>', $result);

        // sql, 2 rows, head, tail, rbeg, rend, fbeg, fend, renr
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>\n10.tail = </table>\n10.rbeg = <tr>\n10.rend = ++\n10.fbeg = <td>\n10.fend = </td>\n10.renr = </tr>");
        $this->assertEquals('<table><tr><td>Doe</td><td>John</td>++</tr><tr><td>Smith</td><td>Jane</td>++</tr></table>', $result);

        // sql, 2 rows, head, tail, rbeg, rend, fbeg, fend, renr, rsep
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>\n10.tail = </table>\n10.rbeg = <tr>\n10.rend = ++\n10.fbeg = <td>\n10.fend = </td>\n10.renr = </tr>\n10.rsep = @");
        $this->assertEquals('<table><tr><td>Doe</td><td>John</td>++</tr>@<tr><td>Smith</td><td>Jane</td>++</tr></table>', $result);

        // sql, 2 rows, head, tail, rbeg, rend, fbeg, fend, renr, rsep, fsep
        $result = $this->report->process("10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2\n10.head = <table>\n10.tail = </table>\n10.rbeg = <tr>\n10.rend = ++\n10.fbeg = <td>\n10.fend = </td>\n10.renr = </tr>\n10.rsep = @\n10.fsep = $$");
        $this->assertEquals('<table><tr><td>Doe</td>$$<td>John</td>++</tr>@<tr><td>Smith</td>$$<td>Jane</td>++</tr></table>', $result);

        // no 'sql'
        $result = $this->report->process("10.tail = </table>\n10.rbeg = <tr>\n10.rend = ++\n10.fbeg = <td>\n10.fend = </td>\n10.renr = </tr>\n10.rsep = @\n10.fsep = $$");
        $this->assertEquals('', $result);

        // head, tail, rbeg, rend, fbeg, fend, renr, rsep, fsep, sql 2 rows
        $result = $this->report->process("10.head = <table>\n10.tail = </table>\n10.rbeg = <tr>\n10.rend = ++\n10.fbeg = <td>\n10.fend = </td>\n10.renr = </tr>\n10.rsep = @\n10.fsep = $$\n10.sql = SELECT name, firstname FROM Person ORDER BY id LIMIT 2");
        $this->assertEquals('<table><tr><td>Doe</td>$$<td>John</td>++</tr>@<tr><td>Smith</td>$$<td>Jane</td>++</tr></table>', $result);
    }

    /**
     *
     */
    public function testReportMultipleQuery() {

        // nested 2
        $result = $this->report->process("10.sql = SELECT id FROM Person ORDER BY id LIMIT 2\n10.10.sql = SELECT name FROM Person ORDER BY id LIMIT 2\n");
        $this->assertEquals('1DoeSmith2DoeSmith', $result);

        // nested 3
        $result = $this->report->process("10.sql = SELECT id FROM Person ORDER BY id LIMIT 2\n10.10.sql = SELECT name FROM Person ORDER BY id LIMIT 2\n\n10.10.10.sql = SELECT firstname FROM Person ORDER BY id LIMIT 2\n");
        $this->assertEquals('1DoeJohnJaneSmithJohnJane2DoeJohnJaneSmithJohnJane', $result);

        // nested 2, seq
        $result = $this->report->process("10.sql = SELECT id FROM Person ORDER BY id LIMIT 2\n10.10.sql = SELECT name FROM Person ORDER BY id LIMIT 2\n\n10.20.sql = SELECT firstname FROM Person ORDER BY id LIMIT 2\n");
        $this->assertEquals('1DoeSmithJohnJane2DoeSmithJohnJane', $result);
    }

    /**
     *
     */
    public function testReportLink() {

        // link
        $result = $this->report->process("10.sql = SELECT 'u:http://www.example.com' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="http://www.example.com" class="external" >http://www.example.com</a>', $result);

        // link, checked
        $result = $this->report->process("10.sql = SELECT 'u:http://www.example.com|C' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="http://www.example.com" ><img alt="Checked green" src="typo3conf/ext/qfq/Resources/Public/icons/checked-green.gif" title="green" ></a>', $result);

        // linck, checked, text
        $result = $this->report->process("10.sql = SELECT 'u:http://www.example.com|C|t:Hello World' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="http://www.example.com" ><img alt="Checked green" src="typo3conf/ext/qfq/Resources/Public/icons/checked-green.gif" title="green" > Hello World</a>', $result);

        // link, checked, text, tooltip
        $result = $this->report->process("10.sql = SELECT 'u:http://www.example.com|C|t:Hello World|o:more information' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="http://www.example.com" title="more information" ><img alt="Checked green" src="typo3conf/ext/qfq/Resources/Public/icons/checked-green.gif" title="green" > Hello World</a>', $result);
    }

    /**
     *
     */
    public function testReportLinkRenderMode() {

        // r:0, url + text
        $result = $this->report->process("10.sql = SELECT 'u:http://www.example.com|t:Hello World|r:0' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="http://www.example.com" class="external" >Hello World</a>', $result);

        // r:0, url
        $result = $this->report->process("10.sql = SELECT 'u:http://www.example.com|r:0' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="http://www.example.com" class="external" >http://www.example.com</a>', $result);

        // r:0, text
        $result = $this->report->process("10.sql = SELECT 'r:0|t:Hello World' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('', $result);

        // r:1, url + text
        $result = $this->report->process("10.sql = SELECT 'u:http://www.example.com|t:Hello World|r:1' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="http://www.example.com" class="external" >Hello World</a>', $result);

        // r:1, url
        $result = $this->report->process("10.sql = SELECT 'u:http://www.example.com|r:1' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="http://www.example.com" class="external" >http://www.example.com</a>', $result);

        // r:1, text
        $result = $this->report->process("10.sql = SELECT 'r:1|t:Hello World' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('Hello World', $result);

    }

    /**
     *
     */
    public function testReportMailto() {

        // r:0, mailto
        $result = $this->report->process("10.sql = SELECT 'm:john.doe@example.com' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="mailto:john.doe@example.com" class="external" >mailto:john.doe@example.com</a>', $result);

        $result = $this->report->process("10.sql = SELECT 'm:john.doe@example.com|e:0' AS _link FROM Person ORDER BY id LIMIT 1");
        $this->assertEquals('<a href="mailto:john.doe@example.com" class="external" >mailto:john.doe@example.com</a>', $result);

        //TODO: enable encryption check
//        $result = $this->report->process("10.sql = SELECT 'm:john.doe@example.com|e:1' AS _link FROM Person ORDER BY id LIMIT 1");
//        $this->assertEquals('<a href="mailto:john.doe@example.com" >mailto:john.doe@example.com</a>', $result);

    }

    /**
     *
     */
    public function testReportPageToken() {

        //
//        $result = $this->report->process("10.sql = SELECT 'p:form&r=123&a=hello&type=5&L=3&final=world|t:Person' AS _page FROM Person ORDER BY id LIMIT 1");
//        $this->assertEquals('', $result);


    }

    /**
     *
     */
    public function testReportPageFix() {

        //
//        $result = $this->report->process("10.sql = SELECT 'form|r=123&a=hello&type=5&L=3&final=world|Person' AS _Page FROM Person ORDER BY id LIMIT 1");
//        $this->assertEquals('', $result);


    }


    /**
     * @throws Exception
     */
    protected function setUp() {

        parent::setUp();
        $GLOBALS["TSFE"] = new FakeTSFEReport();
        $this->eval = new qfq\Evaluate($this->store, $this->db);
        $this->report = new qfq\Report(array(), $this->store->getVar(SYSTEM_SESSION_NAME, STORE_SYSTEM), $this->eval, true);

        $this->executeSQLFile(__DIR__ . '/fixtures/Generic.sql', true);
    }
}

class FakeTSFEReport {
    public $id = 1;
    public $type = 1;
    public $sys_language_uid = 1;
}
